# ES2015 and Babel Configuration

`vue-loader` ships with ES2015 (aka ES6) enabled by default in `<script>` tags. All scripts inside Vue components are compiled with [Babel](https://babeljs.io/) using `babel-loader`. If you haven't picked up the new language features yet, you definitely should. Some good learning resources:

- [Babel - Learn ES2015](https://babeljs.io/docs/learn-es2015/)
- [ES6 Features](https://github.com/lukehoban/es6features)
- [Exploring ES6 (book)](https://leanpub.com/exploring-es6)

Here's a typical pattern you will see when importing other Vue components:

``` html
<script>
import ComponentA from './ComponentA.vue'
import ComponentB from './ComponentB.vue'

export default {
  components: {
    ComponentA,
    ComponentB
  }
}
</script>
```

We are using ES2015's Object literal shorthand here to define the child components. `{ ComponentA }` is simply shorthand for `{ ComponentA: ComponentA }`. Vue will automatically convert the key to `component-a`, so you can use the imported component in the template as `<component-a>`.

### Using Custom Babel Configuration

> **Compatibility Note**: `vue-loader` 7.0+ uses Babel 6. If you need to use Babel 5 for transpiling your code, use vue-loader 6.x.

The default Babel options used for scripts inside Vue components are:

``` json
{
  "presets": ["es2015"],
  "plugins": ["transform-runtime"]
}
```

If you want to use custom presets or plugins, for example, to enable stage-0 language features, install the preset/plugin and then add a `babel` field in your Webpack config:

``` bash
npm install babel-preset-stage-0 --save-dev
```

``` js
// webpack.config.js
module.exports = {
  // other configs...
  babel: {
    // enable stage 0 babel transforms.
    presets: ['es2015', 'stage-0'],
    plugins: ['transform-runtime']
  }
}
```

Alternatively, you can add a `.babelrc` file at the root of your project.

### Compiling `.js` Files with Babel

Since we are already using Babel, most likely you'll want to compile your normal `.js` files too! Here's how to do it in your Webpack config:

``` js
// webpack.config.js
module.exports = {
  // other options ...
  module: {
    loaders: [
      {
        // use vue-loader for *.vue files
        test: /\.vue$/,
        loader: 'vue'
      },
      {
        // use babel-loader for *.js files
        test: /\.js$/,
        loader: 'babel',
        // important: exclude files in node_modules
        // otherwise it's going to be really slow!
        exclude: /node_modules/
      }
    ]
  },
  // if you are using babel-loader directly then
  // the babel config block becomes required.
  babel: {
    presets: ['es2015'],
    plugins: ['transform-runtime']
  }
}
```
